//
//  GMTCMapView.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2022 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#if __has_feature(modules)
@import GoogleMaps;
#else
#import <GoogleMaps/GoogleMaps.h>
#endif



@class GMTCConsumerMapStyleCoordinator;
@class GMTCMapView;
@protocol GMTCMapViewSession;

NS_ASSUME_NONNULL_BEGIN

/**
 * \defgroup GMTCMapViewMarkerType GMTCMapViewMarkerType
 * @{
 */

typedef NS_ENUM(NSUInteger, GMTCMapViewMarkerType) {
  GMTCMapViewMarkerTypeUnknown,
  GMTCMapViewMarkerTypeVehicle,
  GMTCMapViewMarkerTypeTripPickupPoint,
  GMTCMapViewMarkerTypeTripDropoffPoint,
  GMTCMapViewMarkerTypeIntermediateDestination,
};

/**@}*/

#pragma mark - GMTCMapViewDelegate

/** Delegate for events on GMTCMapView. */
@protocol GMTCMapViewDelegate <GMSMapViewDelegate>

@optional

/**
 * Called after the mapVIew initialization completes and the mapView is ready to render a mapView
 * session.
 *
 * @param mapview  The GMTCMapView that generated the event.
 */
- (void)mapViewDidInitialize:(GMTCMapView *)mapview;

/**
 * Called after user tapped on the marker.
 *
 * @param mapView  The GMTCMapView that generated the event.
 * @param mapMarker The user tapped marker.
 * @param markerType The type of the tapped marker.
 * @return YES if this delegate handled the tap event, which prevents the map from performing its
 * default selection behavior, and NO if the map should continue with its default selection
 * behavior.
 */
- (BOOL)mapView:(GMSMapView *)mapView
    didTapConsumerMarker:(GMSMarker *)mapMarker
              markerType:(GMTCMapViewMarkerType)markerType;

@end

#pragma mark - GMTCMapView

/**
 * This class represents the main view in the Consumer SDK for iOS.
 * You can use this class to create a customizable map so that your users can
 * use the `consumerMapStylecoordinator` to create custom markers and polylines.
 *
 * Note: You should instantiate this map via the default constructor [[GMTCMapView alloc]
 * initWithFrame:].
 *
 * GMTCMapView can be read and modified only from the main thread, similar to GMSMapView and all
 * UIKit objects. Calling these methods from another thread results in an exception or undefined
 * behavior.
 */
@interface GMTCMapView : GMSMapView

/** Delegate of the map view. */
@property(nonatomic, weak, nullable) id<GMTCMapViewDelegate> delegate;

/** Maintains customizable UI options for the library's markers, polylines, and polygons. */
@property(nonatomic, readonly) GMTCConsumerMapStyleCoordinator *consumerMapStyleCoordinator;

/** Allows the map view's camera to auto-update. The default value is YES. */
@property(nonatomic, getter=isAllowCameraAutoUpdate) BOOL allowCameraAutoUpdate;

/** Initialization of the GMTCMapView object. */
- (instancetype)initWithFrame:(CGRect)frame;

/** Initialization with coder. */
- (nullable instancetype)initWithCoder:(NSCoder *)aDecoder
    __deprecated_msg("Use <code>-initWithFrame</code> instead.");




/**
 * Resets the mapView's camera to the default position based on the current active mapView
 * session(s).
 */
- (void)resetCamera;

/**
 * Forces a redraw of all the overlays for ridesharing that were rendered in the current map view.
 * These overlays include the ridesharing related markers referenced by @c
 * GMTCCustomizableMarkerType, the polygons for pickup compounded area and trip routes. You can
 * use this method to redraw overlays cleaned by calling @c clear on GMTCMapView. Note: This API
 * does not restore the overlays added from an application layer.
 */
- (void)redrawRidesharingOverlays;

/** Shows a mapView session on the mapView. */
- (void)showMapViewSession:(id<GMTCMapViewSession>)session;

/** Hides a mapView session on the mapView. */
- (void)hideMapViewSession:(id<GMTCMapViewSession>)session;

/** Hide all the mapView sessions on the mapView. */
- (void)hideAllMapViewSessions;

/** Returns all the mapView sessions currently shown on the mapView. */
- (NSArray<id<GMTCMapViewSession>> *)activeSessions;

@end

NS_ASSUME_NONNULL_END
