//
//  GMTCMarkerStyleOptions.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2022 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

#import "GMTSImmutableData.h"

@class UIImage;

NS_ASSUME_NONNULL_BEGIN

/**
 * \defgroup GMTCCustomizableMarkerType GMTCCustomizableMarkerType
 * @{
 */

/** Enum definition of all supported customizable marker types. */
typedef NS_ENUM(NSInteger, GMTCCustomizableMarkerType) {
  /** Placemark for internal marker types that are not customizable. */
  GMTCCustomizableMarkerTypeUnknown = -1,

  /** Pickup point icon shown when a trip is in progress with a participating rider. */
  GMTCCustomizableMarkerTypeTripPickupPoint,

  /** Dropoff point icon shown when the trip is in progress with a participating rider. */
  GMTCCustomizableMarkerTypeTripDropoffPoint,

  /**
   * Vehicle icon shown when the trip is in progress. This icon matches the trip connected to the
   * rider.
   */
  GMTCCustomizableMarkerTypeTripVehicle,

  /** An intermediate destination in an active trip. */
  GMTCCustomizableMarkerTypeIntermediateDestination,
};

/**@}*/

/** Data object that holds custom marker properties from the client application. */
@interface GMTCMarkerStyleOptions : GMTSImmutableData <NSCopying, NSMutableCopying>

/** The ground anchor for the marker. Defaults to {0.5f, 0.5f} */
@property(nonatomic, readonly) CGPoint groundAnchor;

/** Boolean that determines the visibility of the marker. Defaults to YES. */
@property(nonatomic, readonly) BOOL isVisible;

/** Marker view to render. If left nil, falls back to the icon property instead. */
@property(nonatomic, readonly, nullable) UIView *iconView;

/** The icon to display for the marker. */
@property(nonatomic, readonly, nullable) UIImage *icon;

/** The z-index specifies the stack order of this marker. Defaults to 0. */
@property(nonatomic, readonly) int32_t zIndex;

/** Whether the marker is flat against the map or a billboard facing the camera. Defaults to NO. */
@property(nonatomic, readonly) BOOL isFlat;

/**
 * Initializes the object with default values: `groundAnchor:{0.5f, 0.5f}`, `isVisible:YES`,
 * `iconView:nil`, `icon:nil`, `zIndex:0`, `isFlat:NO`.
 */
- (instancetype)init;

@end

/**
 * Mutable data object that holds marker's customization properties.
 * Note: When applying the style options for a marker with the
 * `GMTCCustomizableMarkerTypeTripVehicle` type, you must explicitly
 * set the `isFlat` property to YES. Otherwise, when the map is rotated in 3D
 * mode, the vehicle heading won't be aligned with the rotated direction.
 */
@interface GMTCMutableMarkerStyleOptions : GMTCMarkerStyleOptions

/** The ground anchor for the marker. Defaults to {0.5f, 0.5f} */
@property(nonatomic) CGPoint groundAnchor;

/** Boolean that determines the visibility of the marker. Defaults to YES. */
@property(nonatomic) BOOL isVisible;

/** Marker view to render. If left nil, defaults to the icon property value. */
@property(nonatomic, nullable) UIView *iconView;

/**
 * The icon to display for the marker. If there is no iconView, and icon is left nil,
 * the marker's default icon is used.
 */
@property(nonatomic, nullable) UIImage *icon;

/** The z-index specifies the stack order of this marker. Defaults to 0. */
@property(nonatomic) int32_t zIndex;

/** Whether the marker is flat against the map or a billboard facing the camera. Defaults to NO. */
@property(nonatomic) BOOL isFlat;

@end

NS_ASSUME_NONNULL_END
