//
//  GMTCPolylineStyleOptions.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2022 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

#import "GMTSImmutableData.h"
#import "GMTSSpeedReadingSpan.h"

@class GMTCMutablePolylineStyleOptions;

NS_ASSUME_NONNULL_BEGIN

/**
 * \defgroup GMTCPolylineType GMTCPolylineType
 * @{
 */

/** Enum definition of all supported polyline types. */
typedef NS_ENUM(NSUInteger, GMTCPolylineType) {
  GMTCPolylineTypeActiveRoute,
  GMTCPolylineTypeRemainingRoute,
};

/**@}*/

enum GMTSVehicleTypeCategory : NSUInteger;

/** Data object that holds polyline customization properties. */
@interface GMTCPolylineStyleOptions : GMTSImmutableData <NSCopying, NSMutableCopying>

/** The width of the line stroke in screen points. Defaults to 2.0f. */
@property(nonatomic, readonly) CGFloat strokeWidth;

/**
 * Used to specify the color of one or more segments of a polyline. Defaults to [UIColor blueColor].
 */
@property(nonatomic, readonly, copy) UIColor *strokeColor;

/** Boolean that determines the visibility of the polyline. Defaults to YES. */
@property(nonatomic, readonly) BOOL isVisible;

/** Value that determines the z index of the polyline. Defaults to 10. */
@property(nonatomic, readonly) int32_t zIndex;

/**
 * Indicates whether the segments of the polyline should be drawn as geodesics, as opposed to
 * straight lines on the Mercator projection.
 */
@property(nonatomic, readonly) BOOL isGeodesic;

/** Boolean that determines the visibility of the traffic polyline. Defaults to NO. */
@property(nonatomic, readonly) BOOL isTrafficEnabled;

/**
 * Returns the color for the given speed type.
 *
 * @return If traffic is enabled and no color is set, returns the default color for the given speed.
 * If traffic is not enabled, returns the default stroke color.
 */
- (UIColor *)trafficColorForSpeed:(GMTSSpeedType)speedType;

/**
 * Initializes the object with default values: `strokeWidth:2.0f`, `strokeColor:[UIColor
 * blueColor]`, `isVisible:YES`, `zIndex:10`, `isGeodesic:NO`, `isTrafficEnabled:NO`.
 */
- (instancetype)init;

@end

/** Mutable data object that holds polyline customization properties. */
@interface GMTCMutablePolylineStyleOptions : GMTCPolylineStyleOptions

/**
 * The width of the line stroke in screen points. If an invalid value is set here (less than 0.0f),
 * then it will be set to the default value of 2.0f.
 */
@property(nonatomic) CGFloat strokeWidth;

/**
 * Used to specify the color of one or more segments of a polyline. If set to NULL, then it will be
 * set to the default value of [UIColor blueColor].
 */
@property(nonatomic, copy, null_resettable) UIColor *strokeColor;

/** Boolean that determines the visibility of the polyline. Defaults to YES. */
@property(nonatomic) BOOL isVisible;

/** Value that determines the z index of the polyline. Defaults to 10. */
@property(nonatomic) int32_t zIndex;

/**
 * Indicates whether the segments of the polyline should be drawn as geodesics, as
 * opposed to straight lines on the Mercator projection.
 */
@property(nonatomic) BOOL isGeodesic;

/** Boolean that determines the visibility of the traffic polyline. Defaults to NO. */
@property(nonatomic) BOOL isTrafficEnabled;

/**
 * Sets the color for the given speed type. If color is set to NULL, then the
 * default color for the speed type is used.
 *
 * @param speedType The speed type to apply the color to.
 * @param color The color to apply to the speed type.
 */
- (void)setTrafficColorForSpeed:(GMTSSpeedType)speedType color:(nullable UIColor *)color;

@end

NS_ASSUME_NONNULL_END
