//
//  GMTCTripModel.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2022 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import <Foundation/Foundation.h>

#import "GMTSTripPropertyField.h"

@class GMTCMutableTripModelOptions;
@class GMTCTripModel;
@class GMTSLatLng;
@class GMTSTerminalLocation;
@class GMTSTrafficData;
@class GMTSTrip;
@class GMTSTripWaypoint;
@class GMTSVehicleLocation;

enum GMTSTripStatus : NSUInteger;

NS_ASSUME_NONNULL_BEGIN

/**
 * \defgroup GMTCTripModelState GMTCTripModelState
 * @{
 */

/** Describes the activity status of a trip session. */
typedef NS_ENUM(NSUInteger, GMTCTripModelState) {
  /** The trip session is not yet started or stopped. */
  GMTCTripModelStateInactive,

  /** The trip session is started. */
  GMTCTripModelStateActive,
};

/**@}*/

/** A subscriber protocol for GMTCTripModel's event update. */
@protocol GMTCTripModelSubscriber <NSObject>
@optional

/**
 * Called when the trip session state has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param sessionState  The new session state.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateModelState:(enum GMTCTripModelState)sessionState;

/**
 * Called when the current trip has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param trip  The trip that has changed.
 * @param updatedPropertyFields  The updated properties.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
            didUpdateTrip:(nullable GMTSTrip *)trip
    updatedPropertyFields:(GMTSTripPropertyFields)updatedPropertyFields;

/**
 * Called when the current trip status has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param tripStatus  The updated trip status.
 */
- (void)tripModel:(GMTCTripModel *)tripModel didUpdateTripStatus:(enum GMTSTripStatus)tripStatus;

/**
 * Called when the vehicle's location has been updated. The new location may not be different from
 * the previous one.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param vehicleLocation  The last known vehicle location of the current trip.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateVehicleLocation:(nullable GMTSVehicleLocation *)vehicleLocation;

/**
 * Called when the pickup location has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param pickupLocation  The current pickup location. This could be nil if the trip was reset
 * or cancelled.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdatePickupLocation:(nullable GMTSTerminalLocation *)pickupLocation;

/**
 * Called when the dropoff location has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param dropoffLocation  The current dropoff location. This could be nil if the trip was reset
 * or cancelled.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateDropoffLocation:(nullable GMTSTerminalLocation *)dropoffLocation;

/**
 * Called when the ETA to the next waypoint is updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param nextWaypointETA  The driver's current ETA to the pickup point. This could be
 * 0 if the trip was reset or cancelled.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateETAToNextWaypoint:(NSTimeInterval)nextWaypointETA;

/**
 * Called when the driver's ETA to the pickup point is updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param pickupETA  The driver's current ETA to the pickup point. This could be 0 if the trip
 * was either reset or cancelled.
 */
- (void)tripModel:(GMTCTripModel *)tripModel didUpdatePickupETA:(NSTimeInterval)pickupETA;

/**
 * Called when the driver's ETA to the dropoff point is updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param dropoffETA  The driver's current ETA to the drop-off point. This could be 0
 * if the trip was either reset or cancelled.
 */
- (void)tripModel:(GMTCTripModel *)tripModel didUpdateDropoffETA:(NSTimeInterval)dropoffETA;

/**
 * Called when the remaining distance of the trip's active route has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param activeRouteRemainingDistance  Distance in meters that remains on the active route between
 * the current location of the vehicle and the next significant location. The next significant
 * location depends on the status of the current trip. If the trip status is
 * `GMTSTripStatusEnrouteToPickup`, this is the distance to the pickup location. If the trip status
 * is `GMTSTripStatusEnrouteToDropoff`, this is the distance to the drop off location.
 * Your app must provide any necessary measurement conversions.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateActiveRouteRemainingDistance:(int32_t)activeRouteRemainingDistance;

/**
 * Called when the active route segment of the trip has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param activeRoute  The active route segment of the current trip. This segment includes the
 * route from the current location of the vehicle to the next significant location. If the trip
 * status is `GMTSTripStatusEnrouteToPickup`, the next significant location is the pickup
 * location. If the trip status is `GMTSTripStatusEnrouteToDropoff`, the next significant
 * location is the drop off location.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateActiveRoute:(nullable NSArray<GMTSLatLng *> *)activeRoute;

/**
 * Called when the remaining route segment of the trip has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param remainingRoute  The remaining route segment of the current trip. This segment includes
 * the route from the current user's next destination to the user's final destination. If the trip
 * status is `GMTSTripStatusEnrouteToPickup`, the remaining route segment will be the segment from
 * the pickup location to the drop-off location.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateRemainingRoute:(nullable NSArray<GMTSLatLng *> *)remainingRoute;

/**
 * Called when the distance of the trip's remaining route has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param remainingRouteDistance  Distance in meters of the remaining route between the
 * current user's next destination to their final destination. The next destination
 * depends on the status of the current trip. If the trip status is
 * `GMTSTripStatusEnrouteToPickup`, the next destination is the drop-off location.
 * Your app must provide any necessary measurement conversions.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateRemainingRouteDistance:(int32_t)remainingRouteDistance;

/**
 * Called when the trip's intermediate waypoints have been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param remainingWaypoints The current remaining waypoints.
 **/
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateRemainingWaypoints:(nullable NSArray<GMTSTripWaypoint *> *)remainingWaypoints;

/**
 * Called when the trip's waypoints have been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param intermediateDestinations The current remaining intermediate waypoints for this trip.
 **/
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateIntermediateDestinations:
        (nullable NSArray<GMTSTerminalLocation *> *)intermediateDestinations;

/**
 * Called when traffic data has been updated for the active route segment of the trip.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param activeRouteTraffic  The active route traffic segment of the current trip. This segment
 * includes the route from the current location of the vehicle to the next significant location. If
 * the trip status is `GMTSTripStatusEnrouteToPickup`, the next significant location is the pickup
 * location. If the trip status is `GMTSTripStatusEnrouteToDropoff`, the next significant
 * location is the drop off location.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateActiveRouteTraffic:(nullable GMTSTrafficData *)activeRouteTraffic;

/**
 * Called when traffic data has been updated for the remaining route segment of the trip.
 *
 * @param tripModel The instance of GMTCTripModel that generated the event.
 * @param remainingRouteTraffic The traffic data for the remaining route segment of the current
 * trip.
 */
- (void)tripModel:(GMTCTripModel *)tripModel
    didUpdateRemainingRouteTraffic:(nullable GMTSTrafficData *)remainingRouteTraffic;

/**
 * Called when the trip monitoring request error has been updated.
 *
 * @param tripModel  The instance of GMTCTripModel that generated the event.
 * @param error  The error of the current trip monitoring request. The value could be nil.
 */
- (void)tripModel:(GMTCTripModel *)tripModel didFailUpdateTripWithError:(nullable NSError *)error;
@end

/**
 * A class representing a trip session. A trip sesion will provide trip info update periodically
 * after started, it can be either a ridesharing or a delivery trip, and each trip session is
 * identified by a TripName.
 */
@interface GMTCTripModel : NSObject

/** Returns the current session state. */
@property(nonatomic, readonly) GMTCTripModelState state;

/** Returns the current trip data. */
@property(nonatomic, nullable, readonly) GMTSTrip *currentTrip;

/** Returns the associated trip name. */
@property(nonatomic, readonly) NSString *tripName;

/**
 * Specifies the options of the current trip session. This property should only be accessed in the
 * main thread.
 */
@property(nonatomic, readonly) GMTCMutableTripModelOptions *options;

/**
 * Register an instance of GMTCTripModelSubscriber for this session.
 * Call this API before you can get any callbacks.
 * This method is only allowed to be called in the main thread.
 *
 * Note that the tripModel will be active (e.g. trip data is being updated/refreshed) automatically
 * once a subscriber is registered.
 *
 * @param subscriber  The instance of GMTCTripModelSubscriber to be registered.
 */
- (void)registerSubscriber:(id<GMTCTripModelSubscriber>)subscriber;

/**
 * Unregister an instance of GMTCTripModelSubscriber for this session, for avoiding the callback.
 * This is recommended when the instance of GMTCTripModelSubscriber needs to be dealloced.
 * This method is only allowed to be called in the main thread.
 *
 * Note that if there are no subscribers registered, the tripModel will automatically stop the trip
 * session.
 *
 * @param subscriber  The instance of GMTCTripModelSubscriber to be registered.
 */
- (void)unregisterSubscriber:(id<GMTCTripModelSubscriber>)subscriber;

/** Use @c tripModelForTripName: of GMTCTripService instead. */
- (instancetype)init NS_UNAVAILABLE;

@end

NS_ASSUME_NONNULL_END
