//
//  GMTSTrip.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2021 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import "GMTSFleetEngineIDString.h"
#import "GMTSImmutableData.h"
#import "GMTSTripPropertyField.h"

@class GMTSLatLng;
@class GMTSTerminalLocation;
@class GMTSTimestamp;
@class GMTSTrafficData;
@class GMTSTripWaypoint;
@class GMTSVehicleLocation;

NS_ASSUME_NONNULL_BEGIN

/**
 * \defgroup GMTSTripType GMTSTripType
 * @{
 */

/**
 * Defines all the trip types.
 */
typedef NS_ENUM(NSUInteger, GMTSTripType) {
  GMTSTripTypeUnknown,
  GMTSTripTypeShared,
  GMTSTripTypeExclusive,
};

/**@}*/

/**
 * \defgroup GMTSTripStatus GMTSTripStatus
 * @{
 */

/**
 * Defines all the possible trip statuses.
 */
typedef NS_ENUM(NSUInteger, GMTSTripStatus) {
  GMTSTripStatusUnknown,
  GMTSTripStatusNew,
  GMTSTripStatusEnrouteToPickup,
  GMTSTripStatusArrivedAtPickup,
  GMTSTripStatusEnrouteToIntermediateDestination,
  GMTSTripStatusArrivedAtIntermediateDestination,
  GMTSTripStatusEnrouteToDropoff,
  GMTSTripStatusComplete,
  GMTSTripStatusCanceled,
};

/**@}*/

/**
 * Object that contains logistical information about a trip such as
 * `vehicleId`, type of trip, pickup or dropoff times, and route.
 */
@interface GMTSTrip : GMTSImmutableData

/**
 * Name of the trip.
 */
@property(nonatomic, readonly, nullable) NSString *tripName;

/**
 * ID of vehicle servicing the trip.
 */
@property(nonatomic, readonly, nullable) GMTSFleetEngineIDString *vehicleID;

/**
 * The current trip type, see GMTSTripType for all the options.
 */
@property(nonatomic, readonly) GMTSTripType tripType;

/**
 * The current trip status, GMTSTripStatus listed for all the possible statuses.
 */
@property(nonatomic, readonly) GMTSTripStatus tripStatus;

/**
 * The pickup location of the trip.
 */
@property(nonatomic, readonly, nullable) GMTSTerminalLocation *pickupLocation;

/**
 * The estimated time of arrival at the pickup location.
 */
@property(nonatomic, readonly) NSTimeInterval pickupTime;

/**
 * The drop off location of the trip.
 */
@property(nonatomic, readonly, nullable) GMTSTerminalLocation *dropoffLocation;

/**
 * The estimated time of arrival at the drop off location.
 */
@property(nonatomic, readonly) NSTimeInterval dropoffTime;

/**
 * The last location information of the vehicle for the trip.
 */
@property(nonatomic, readonly, nullable) GMTSVehicleLocation *lastVehicleLocation;

/**
 * The number of passengers allowed for the current trip.
 */
@property(nonatomic, readonly) NSUInteger numberOfPassengers;

/**
 * Array of waypoints on the trip route.
 */
@property(nonatomic, readonly, nullable) NSArray<GMTSTripWaypoint *> *remainingWaypoints;

/**
 * The array of the consumer's remaining waypoints.
 */
@property(nonatomic, readonly, nullable) NSArray<GMTSTerminalLocation *> *intermediateDestinations;

/**
 * Index of the intermediate waypoint that the vehicle will arrive at next. If there are no
 * intermediate waypoints, this field is ignored.
 */
@property(nonatomic, readonly) int32_t intermediateDestinationIndex;

/** Returns the tripID for the current trip, converted from tripName. */
- (nullable GMTSFleetEngineIDString *)tripID;

/** Returns the first waypoint in the remaining waypoints list. */
- (nullable GMTSTripWaypoint *)nextTripWaypoint;

/**
 * Returns the next waypoint to be visited. The next waypoint might not belong
 * to this trip if this trip is chained or shared.
 */
- (nullable GMTSTripWaypoint *)nextWaypoint;

/**
 * Returns the active route of the given trip.
 * This route is from the current location of the vehicle to the current user's
 * next destination. Unlike the next waypoint in a back-to-back or carpool trip, the next waypoint
 * may not be related to the current user. The return value is generated the first time it is
 * accessed, and then cached.
 */
- (nullable NSArray<GMTSLatLng *> *)activeRoute;

/**
 * Returns the remaining route of the given trip.
 * The remaining route is the route segment from the current user's next destination to the
 * user's final destination.
 */
- (nullable NSArray<GMTSLatLng *> *)remainingRoute;

/**
 * Returns the traffic data for the active route of this trip.
 * This route is from the current location of the vehicle to the current user's
 * next destination. Unlike the next waypoint in a back-to-back or carpool trip, the next waypoint
 * may not be related to the current user. The return value is generated the first time it is
 * accessed, and then cached.
 */
- (nullable GMTSTrafficData *)activeRouteTraffic;

/**
 * Returns the traffic data for the remaining route of this trip.
 * The remaining route is the route segment from the current user's next destination to the
 * user's final destination.
 */
- (nullable GMTSTrafficData *)remainingRouteTraffic;

/**
 * Returns the remaining distance of the active route.
 * The active route means the route segment from the vehicle's location to the current user's next
 * destination.
 */
- (int32_t)remainingDistanceOfActiveRouteInMeters;

/**
 * Returns the distance of the remaining route.
 * The remaining route means the route segment from the current user's next destination to their
 * last destination.
 */
- (int32_t)distanceOfRemainingRouteInMeters;

/**
 * Calculate the difference between two given trips. Each property difference will be accumulated
 * into the return result.
 *
 * @param trip1  The first trip candidate.
 * @param trip2  The second trip candidate.
 */
+ (GMTSTripPropertyFields)propertyValueDifferencesBetweenTrip1:(GMTSTrip *)trip1
                                                      andTrip2:(GMTSTrip *)trip2;

/**
 * Instantiates an instance of this class.
 *
 * @param tripName  Name of the trip.
 * @param vehicleID  The ID of vehicle servicing the trip.
 * @param tripType  Trip type, see enum GMTSTripType for all the options.
 * @param tripStatus  Trip status, see enum GMTSTripStatus for all the possible statuses.
 * @param pickupLocation  The target pickup location.
 * @param pickupTime  The estimated time for arriving at the pickup location.
 * @param dropoffLocation  The target drop off location.
 * @param dropoffTime  The estimated time for arriving at the drop off location.
 * @param lastVehicleLocation  The last vehicle location information.
 * @param numberOfPassengers The number of passengers allowed in the trip.
 * @param remainingWaypoints The array of waypoints remaining on the trip.
 * @param intermediateDestinations The array of the consumer's remaining waypoints.
 * @param intermediateDestinationIndex Index of the intermediate waypoint that the vehicle will
          arrive at next.
 */
- (instancetype)initWithTripName:(nullable NSString *)tripName
                       vehicleID:(nullable GMTSFleetEngineIDString *)vehicleID
                        tripType:(GMTSTripType)tripType
                      tripStatus:(GMTSTripStatus)tripStatus
                  pickupLocation:(nullable GMTSTerminalLocation *)pickupLocation
                      pickupTime:(NSTimeInterval)pickupTime
                 dropoffLocation:(nullable GMTSTerminalLocation *)dropoffLocation
                     dropoffTime:(NSTimeInterval)dropoffTime
             lastVehicleLocation:(nullable GMTSVehicleLocation *)lastVehicleLocation
              numberOfPassengers:(NSUInteger)numberOfPassengers
              remainingWaypoints:(nullable NSArray<GMTSTripWaypoint *> *)remainingWaypoints
        intermediateDestinations:
            (nullable NSArray<GMTSTerminalLocation *> *)intermediateDestinations
    intermediateDestinationIndex:(int32_t)intermediateDestinationIndex;

/**
 * Use @c initWithTripName:vehicleID:tripType:tripStatus:pickupLocation:pickupTime:dropoffLocation:
 * dropoffTime:route:lastVehicleLocation:numberOfPassengers:remainingWaypoints:
 * intermediateDestinations:intermediateDestinationIndex: instead.
 */
- (instancetype)init NS_UNAVAILABLE;

@end

NS_ASSUME_NONNULL_END
