//
//  GMTSTripWaypoint.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2022 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import "GMTSFleetEngineIDString.h"
#import "GMTSImmutableData.h"

@class GMTSLatLng;
@class GMTSTerminalLocation;
@class GMTSTimestamp;
@class GMTSTrafficData;

NS_ASSUME_NONNULL_BEGIN

/**
 * \defgroup GMTSTripWaypointType GMTSTripWaypointType
 * @{
 */

/**
 * Enum defining the waypoints associated with a trip.
 */
typedef NS_ENUM(NSUInteger, GMTSTripWaypointType) {
  GMTSTripWaypointTypeUnknown,
  GMTSTripWaypointTypePickUp,
  GMTSTripWaypointTypeDropOff,
  GMTSTripWaypointTypeIntermediateDestination,
};

/**@}*/

/**
 * Object that describes a vehicle stop on its route.
 * For a trip with multiple passengers,
 * this point might be an intermediate stop or the end of a trip.
 */
@interface GMTSTripWaypoint : GMTSImmutableData

/**
 * The location associated with the 'tripWaypoint'.
 */
@property(nonatomic, readonly, nullable) GMTSTerminalLocation *location;

/**
 * The trip ID associated with the waypoint.
 */
@property(nonatomic, readonly, nullable) GMTSFleetEngineIDString *tripID;

/**
 * The type of the trip waypoint.
 */
@property(nonatomic, readonly) GMTSTripWaypointType waypointType;

/**
 * The path distance calculated by the Fleet Engine from the previous waypoint on a path to
 * the current waypoint.
 */
@property(nonatomic, readonly) int32_t distanceToPreviousWaypointInMeters;

/**
 * The estimated time of arrival to this waypoint.
 */
@property(nonatomic, readonly) NSTimeInterval ETA;

/**
 * The path calculated by the server from the previous waypoint
 * to the current waypoint encoded as an array of LatLng.
 */
@property(nonatomic, readonly, nullable) NSArray<GMTSLatLng *> *routeFromPreviousWaypoint;

/**
 * The traffic data for the path leading to this waypoint.
 */
@property(nonatomic, readonly, nullable) GMTSTrafficData *trafficData;

/**
 * Instantiates the object.
 *
 * @param location  The latitude and longitude associated with the waypoint.
 * @param tripID  The ID of the trip which associated with the waypoint.
 * @param waypointType  The type of terminal associated with the waypoint.
 * @param distanceToPreviousWaypointInMeters  The path distance calculated by Fleet Engine from the
 * previous waypoint to the current waypoint.
 * @param ETA  The estimated time of arrival to the waypoint.
 */
- (instancetype)initWithLocation:(nullable GMTSTerminalLocation *)location
                                tripID:(nullable GMTSFleetEngineIDString *)tripID
                          waypointType:(GMTSTripWaypointType)waypointType
    distanceToPreviousWaypointInMeters:(int32_t)distanceToPreviousWaypointInMeters
                                   ETA:(NSTimeInterval)ETA;

/**
 * Use @c initWithLocation:tripID:waypointType:distanceMeters:eta: instead.
 */
- (null_unspecified instancetype)init NS_UNAVAILABLE;

@end

NS_ASSUME_NONNULL_END
