//
//  GMTSVehicle.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2021 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import "GMTSImmutableData.h"
#import "GMTSVehicleSupportedTripType.h"

@class GMTSVehicleLocation;
@class GMTSVehicleType;

NS_ASSUME_NONNULL_BEGIN

/**
 * \defgroup GMTSVehicleState GMTSVehicleState
 * @{
 */

/**
 * Enum that defines all vehicle states.
 */
typedef NS_ENUM(NSUInteger, GMTSVehicleState) {
  GMTSVehicleStateUnknown,
  GMTSVehicleStateOffline,
  GMTSVehicleStateOnline,
};

/**@}*/

/**
 * \defgroup GMTSVehicleSearchOrder GMTSVehicleSearchOrder
 * @{
 */

/**
 * Enum that defines all supported search orders.
 */
typedef NS_ENUM(NSUInteger, GMTSVehicleSearchOrder) {
  GMTSVehicleSearchOrderUnknown,
  GMTSVehicleSearchOrderByPickupETA,
  GMTSVehicleSearchOrderByPickupDistance,
  GMTSVehicleSearchOrderByDropoffETA,
  GMTSVehicleSearchOrderByPickupPointStraightDistance,
  GMTSVehicleSearchOrderByCost,
};

/**@}*/

#pragma mark - GMTSVehicleAttributeKeyValuePair

/**
 * Object that contains a vehicle attribute key/value pair.
 */
@interface GMTSVehicleAttributeKeyValuePair : GMTSImmutableData

/**
 * The key of this pair.
 */
@property(nonatomic, readonly, copy) NSString *key;

/**
 * The value of this pair.
 */
@property(nonatomic, readonly, copy) NSString *value;

/**
 * Initialization of the object.
 *
 * @param key  The key, should not be nil.
 * @param value  The value, should not be nil.
 */
- (instancetype)initWithKey:(NSString *)key value:(NSString *)value;

/**
 * Will return nil.
 * Use @c initWithKey:value: instead.
 */
- (instancetype)init NS_UNAVAILABLE;

@end

#pragma mark - GMTSVehicle

/**
 * Object that contains functional data related to a vehicle. This data can be profile data, such as
 * the max capacity, or tracking data, such as current trips and location.
 */
@interface GMTSVehicle : GMTSImmutableData

/**
 * The name of the vehicle.
 */
@property(nonatomic, readonly, copy, nullable) NSString *vehicleName;

/**
 * The vehicle state, see GMTSVehicleState for all the options.
 */
@property(nonatomic, readonly) GMTSVehicleState vehicleState;

/**
 * Supported trip types, see GMTSVehicleSupportedTripTypes for all the options.
 */
@property(nonatomic, readonly) GMTSVehicleSupportedTripTypes supportedTripTypes;

/**
 * An array contains all the current trip ID.
 */
@property(nonatomic, readonly, copy, nullable) NSArray<NSString *> *currentTrips;

/**
 * The last location of the vehicle.
 */
@property(nonatomic, readonly, copy, nullable) GMTSVehicleLocation *lastLocation;

/**
 * The maximum capacity.
 */
@property(nonatomic, readonly) int32_t maximumCapacity;

/**
 * An array contains all the vehicle attributes.
 */
@property(nonatomic, readonly, copy, nullable)
    NSArray<GMTSVehicleAttributeKeyValuePair *> *attributes;

/**
 * The vehicle type information.
 */
@property(nonatomic, readonly, copy) GMTSVehicleType *vehicleType;

/**
 * Initialization of the object.
 *
 * @param vehicleName  The name of the vehicle.
 * @param vehicleState  The vehicle state, see GMTSVehicleState for all the options.
 * @param supportedTripTypes  All the supported trip types.
 * @param currentTrips  An array contains all the current trip ID.
 * @param lastLocation  The last location of the vehicle.
 * @param maximumCapacity  The maximum capacity.
 * @param attributes  A dictionary contains all the vehicle attributes.
 * @param vehicleType  The vehicle type information.
 */
- (instancetype)initWithvehicleName:(nullable NSString *)vehicleName
                       vehicleState:(GMTSVehicleState)vehicleState
                 supportedTripTypes:(GMTSVehicleSupportedTripTypes)supportedTripTypes
                       currentTrips:(nullable NSArray<NSString *> *)currentTrips
                       lastLocation:(nullable GMTSVehicleLocation *)lastLocation
                    maximumCapacity:(int32_t)maximumCapacity
                         attributes:
                             (nullable NSArray<GMTSVehicleAttributeKeyValuePair *> *)attributes
                        vehicleType:(GMTSVehicleType *)vehicleType;

/**
 * Will return nil.
 * Use @c initWithvehicleName:vehicleState:supportedTripTypes:currentTrips:lastLocation:
 * maximumCapacity:attributes:vehicleType: instead.
 */
- (instancetype)init NS_UNAVAILABLE;

@end

NS_ASSUME_NONNULL_END
