//
//  GMTSVehicleLocation.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2021 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import <CoreLocation/CoreLocation.h>

#import "GMTSImmutableData.h"

@class GMTSLatLng;
@class GMTSTimestamp;

NS_ASSUME_NONNULL_BEGIN

/**
 * Object that contains the location information for the vehicle that describes
 * the heading, speed, and time when this information was retrieved.
 */
@interface GMTSVehicleLocation : GMTSImmutableData

/**
 * Location of the vehicle.
 */
@property(nonatomic, readonly, nullable) GMTSLatLng *latLng;

/**
 * The radius of uncertainty for |latLng|, measured in meters.  The
 * |latLng| indicates the center of the circle, and |latLngAccuracy| indicates the
 * radius of that circle.
 */
@property(nonatomic, readonly) double latLngAccuracy;

/**
 * Heading corner radius of the vehicle.
 */
@property(nonatomic, readonly) CLLocationDirection heading;

/**
 * The radius of uncertainty for |heading|, measured in degrees.  The |heading|
 * identifies the mean of the heading estimate and this value indicates the
 * standard deviation of the estimate.
 */
@property(nonatomic, readonly) double headingAccuracy;

/**
 * Speed of the vehicle.
 */
@property(nonatomic, readonly) int32_t speed;

/**
 * Accuracy of speed in meters per second.
 */
@property(nonatomic, readonly) double speedAccuracy;

/**
 * The timestamp of the update time.
 */
@property(nonatomic, readonly) NSTimeInterval updateTime;

/**
 * Indicates whether the vehicle can be snapped to the route.
 */
@property(nonatomic, readonly) BOOL isSnappableToRoute;

/**
 * Designated initialization.
 *
 * @param latLng  The location of the vehicle.
 * @param latLngAccuracy The  location accuracy of the vehicle.
 * @param heading  The heading corner radius of the vehicle.
 * @param headingAccuracy The heading accuracy of the vehicle.
 * @param speed The speed of the vehicle.
 * @param speedAccuracy The speed accuracy of the vehicle.
 * @param updateTime The timestamp of the update time.
 * @param isSnappableToRoute Indicates whether the vehicle can be snapped to the route.
 */
- (instancetype)initWithLatLng:(nullable GMTSLatLng *)latLng
                latLngAccuracy:(double)latLngAccuracy
                       heading:(CLLocationDirection)heading
               headingAccuracy:(double)headingAccuracy
                         speed:(int32_t)speed
                 speedAccuracy:(double)speedAccuracy
                    updateTime:(NSTimeInterval)updateTime
            isSnappableToRoute:(BOOL)isSnappableToRoute NS_DESIGNATED_INITIALIZER;

/**
 * Use @c
 * initWithLatLng:latLngAccuracy:heading:headingAccuracy:speed:speedAccuracy:updateTime:isSnappableToRoute:
 * instead.
 */
- (instancetype)init NS_UNAVAILABLE;

@end

NS_ASSUME_NONNULL_END
